
# type(scope)!: description
# types: fix, feat, build, chore, ci, docs, style, refactor, perf, test
# scope: optional name of component, environment, or release
# '!': optional switch to signal a breaking change
# description: short description

# [optional body]
# Optional long description usefull for diagnosis

# [optional footer(s)]
# Reviewed-by: Full Name <email>
# Refs #NUMBER
# Fix #NUMBER
# BREAKING CHANGE: This can break things
