# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide exceptions."""


class PromanVersioningException(Exception):
    """Provide base errors in PackageManager."""
